PROJECT_ID_ENV_VAR = "MANTIK_PROJECT_ID"
DATA_REPOSITORY_ID_ENV_VAR = "MANTIK_DATA_REPOSITORY_ID"
DATA_REPOSITORY_VERSION_ENV_VAR = "MANTIK_DATA_REPOSITORY_VERSION"
DATA_REPOSITORY_COMMIT_ENV_VAR = "MANTIK_DATA_REPOSITORY_COMMIT"
DATA_REPOSITORY_BRANCH_ENV_VAR = "MANTIK_DATA_REPOSITORY_BRANCH"
EXPERIMENT_REPOSITORY_ID_ENV_VAR = "MANTIK_EXPERIMENT_REPOSITORY_ID"
CODE_REPOSITORY_ID_ENV_VAR = "MANTIK_CODE_REPOSITORY_ID"
MODEL_ID_ENV_VAR = "MANTIK_MODEL_ID"
CONNECTION_ID_ENV_VAR = "MANTIK_CONNECTION_ID"
RUN_ID_ENV_VAR = "MANTIK_RUN_ID"
MLFLOW_RUN_ID_ENV_VAR = "MLFLOW_RUN_ID"
TARGET_DIR_ENV_VAR = "MANTIK_TARGET_DIR"
MANTIK_ACCESS_TOKEN_ENV_VAR = "MLFLOW_TRACKING_TOKEN"
